/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'bin'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_bin(struct_processus *s_etat_processus)
{
	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  BIN ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(base binaire)\n\n");
			printf("  Aucun argument\n");
		}
		else
		{
			printf("(binary base)\n\n");
			printf("  No argument\n");
		}

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 0) == d_erreur)
		{
			return;
		}
	}

	cf(s_etat_processus, 43);
	sf(s_etat_processus, 44);

	return;
}


/*
================================================================================
  Fonction 'beep'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_beep(struct_processus *s_etat_processus)
{
	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  BEEP ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(son d'avertissement)\n\n");
			printf("  Aucun argument\n");
		}
		else
		{
			printf("(warning bell)\n\n");
			printf("  No argument\n");
		}

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 0) == d_erreur)
		{
			return;
		}
	}

	if (test_cfsf(s_etat_processus, 51) == d_faux)
	{
		printf("%s", ds_beep);
	}

	return;
}


/*
================================================================================
  Fonction 'b->r'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_b_vers_r(struct_processus *s_etat_processus)
{
	struct_objet				*s_objet_argument;
	struct_objet				*s_objet_resultat;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  B->R ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(binaire vers rel)\n\n");
		}
		else
		{
			printf("(binary to real)\n\n");
		}

		printf("    1: %s\n", d_BIN);
		printf("->  1: %s\n", d_INT);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*s_objet_argument).type == BIN)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*s_objet_resultat).objet)) = (integer8) (*((logical8 *)
				(*s_objet_argument).objet));
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	return;
}


/*
================================================================================
  Fonction 'backspace'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_backspace(struct_processus *s_etat_processus)
{
	struct_descripteur_fichier	*descripteur;

	integer8					i;
	integer8					nombre_octets;
	integer8					position_finale;
	integer8					position_initiale;
	integer8					saut;
	integer8					pointeur;
	integer8					niveau;
	integer8					longueur_effective;
	integer8					longueur_questure;


	logical1					guillemets_a_cheval;
	logical1					presence_chaine;
	logical1					presence_indicateur;

	struct flock				lock;

	struct_objet				*s_objet_argument;

	unsigned char				*tampon_lecture;
	unsigned char				tampon[9];

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  BACKSPACE ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(retour  l'enregistrement prcdent)\n\n");
		}
		else
		{
			printf("(return to the previous record)\n\n");
		}

		printf("    1: %s\n", d_FCH);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*s_objet_argument).type == FCH)
	{
		/*
		 * Fichiers  accs squentiel
		 */

		if ((*((struct_fichier *) (*s_objet_argument).objet)).acces == 'S')
		{
			/*
			 * Vrification des verrous
			 */

			lock.l_type = F_RDLCK;
			lock.l_whence = SEEK_SET;
			lock.l_start = 0;
			lock.l_len = 0;
			lock.l_pid = getpid();

			if ((descripteur = descripteur_fichier(s_etat_processus,
					(struct_fichier *) (*s_objet_argument).objet)) == NULL)
			{
				liberation(s_etat_processus, s_objet_argument);
				return;
			}

			if (fcntl(fileno((*descripteur).descripteur_c), F_GETLK, &lock)
					== -1)
			{
				liberation(s_etat_processus, s_objet_argument);

				(*s_etat_processus).erreur_systeme = d_es_erreur_fichier;
				return;
			}

			if (lock.l_type != F_UNLCK)
			{
				liberation(s_etat_processus, s_objet_argument);

				(*s_etat_processus).erreur_execution =
						d_ex_fichier_verrouille;
				return;
			}

			if ((*((struct_fichier *) (*s_objet_argument).objet)).binaire
					== 'N')
			{
				/*
				 * Fichiers formats
				 */

				if ((position_finale = ftell((*descripteur).descripteur_c))
						== -1)
				{
					liberation(s_etat_processus, s_objet_argument);

					(*s_etat_processus).erreur_systeme = d_es_erreur_fichier;
					return;
				}

				longueur_questure = 256;

				if ((tampon_lecture = malloc(((size_t) longueur_questure) *
						sizeof(unsigned char))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				do
				{
					if ((position_initiale = (position_finale -
							longueur_questure)) < 0)
					{
						position_initiale = 0;
						longueur_effective = position_finale + 1;
					}
					else
					{
						longueur_effective = longueur_questure;
					}

					if (fseek((*descripteur).descripteur_c,
							(long) position_initiale, SEEK_SET) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_erreur_fichier;
						return;
					}

					longueur_effective = (integer8) fread(tampon_lecture,
							sizeof(unsigned char), (size_t) longueur_effective,
							(*descripteur).descripteur_c);

					pointeur = longueur_effective - 1;
					presence_indicateur = d_faux;

					while((pointeur >= 0) && (presence_indicateur == d_faux))
					{
						if (tampon_lecture[pointeur] == '}')
						{
							presence_indicateur = d_vrai;
						}
						else
						{
							position_finale--;
							pointeur--;
						}
					}
				} while((longueur_effective == longueur_questure) &&
						(presence_indicateur == d_faux));

				if (presence_indicateur == d_faux)
				{
					/*
					 * Le dbut du fichier est atteint.
					 */

					if (fseek((*descripteur).descripteur_c, 0, SEEK_SET) != 0)
					{
						liberation(s_etat_processus, s_objet_argument);
						free(tampon_lecture);

						(*s_etat_processus).erreur_systeme =
								d_es_erreur_fichier;
						return;
					}

					(*s_etat_processus).erreur_execution =
							d_ex_debut_de_fichier_atteint;

					liberation(s_etat_processus, s_objet_argument);
					free(tampon_lecture);

					return;
				}

				position_finale = position_finale - 1;
				presence_chaine = d_faux;
				niveau = 1;

				if (position_finale < 0)
				{
					liberation(s_etat_processus, s_objet_argument);
					free(tampon_lecture);

					(*s_etat_processus).erreur_execution =
							d_ex_debut_de_fichier_atteint;
					return;
				}

				do
				{
					if ((position_initiale = (position_finale -
							longueur_questure)) < 0)
					{
						position_initiale = 0;
						longueur_effective = position_finale + 1;
					}
					else
					{
						longueur_effective = longueur_questure;
						position_finale--;
					}

					if (fseek((*descripteur).descripteur_c,
							(long) position_initiale, SEEK_SET) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_erreur_fichier;
						return;
					}

					longueur_effective = (integer8) fread(tampon_lecture,
							sizeof(unsigned char), (size_t) longueur_effective,
							(*descripteur).descripteur_c);

					pointeur = longueur_effective - 1;
					presence_indicateur = d_faux;
					guillemets_a_cheval = d_faux;

					while((pointeur >= 0) && (presence_indicateur == d_faux)
							&& (guillemets_a_cheval == d_faux))
					{
						if (tampon_lecture[pointeur] == '"')
						{
							if (pointeur > 0)
							{
								// On n'est pas au dbut du buffer, on regarde
								// si les guillemets sont chapps.

								if (tampon_lecture[pointeur - 1] != '\\')
								{
										presence_chaine = (presence_chaine
												== d_vrai) ? d_faux : d_vrai;
								}
							}
							else
							{
								// On est au dbut du buffer. Un guillemet
								// peut-tre chapp par le dernier caractre
								// du buffer prcdent.

								guillemets_a_cheval = d_vrai;
							}
						}
						else
						{
							if (tampon_lecture[pointeur] == '}')
							{
								niveau++;
							}
							else if (tampon_lecture[pointeur] == '{')
							{
								niveau--;
							}
						}

						if (guillemets_a_cheval == d_faux)
						{
							if (niveau == 0)
							{
								presence_indicateur = d_vrai;
							}
							else
							{
								position_finale--;
								pointeur--;
							}
						}
					}
				} while((longueur_effective == longueur_questure) &&
						(presence_indicateur == d_faux));

				if (presence_indicateur == d_faux)
				{
					liberation(s_etat_processus, s_objet_argument);
					free(tampon_lecture);

					(*s_etat_processus).erreur_execution =
							d_ex_fin_de_fichier_atteinte;
					return;
				}

				if (fseek((*descripteur).descripteur_c, (long) position_finale,
						SEEK_SET) != 0)
				{
					liberation(s_etat_processus, s_objet_argument);
					free(tampon_lecture);

					(*s_etat_processus).erreur_systeme = d_es_erreur_fichier;
					return;
				}

				free(tampon_lecture);
			}
			else
			{
				/*
				 * Fichiers non formats
				 */

				/*
				 Chaque enregistrement est termin par un champ
				 * indiquant la longueur totale de cet enregistrement.
				 *
				 * XXXXXXX0								longueur sur 7 bits
				 * XXXX0011 XXXXXXXX XXXX0011			longueur sur 16 bits
				 * LSB(1/2) MSB      LSB(2/2)
				 * XXXX0101 XXXXXXXX XXXXXXXX XXXX0101	longueur sur 24 bits
				 * XXXX0111 XXXXXXXX XXXXXXXX XXXXXXXX
				 *          XXXX0111					longueur sur 32 bits
				 * XXXX1001 XXXXXXXX XXXXXXXX XXXXXXXX
				 *          XXXXXXXX XXXX1001			longueur sur 40 bits
				 * XXXX1011 XXXXXXXX XXXXXXXX XXXXXXXX
				 *          XXXXXXXX XXXXXXXX XXXX1011	longueur sur 48 bits
				 * XXXX1101 XXXXXXXX XXXXXXXX XXXXXXXX
				 *          XXXXXXXX XXXXXXXX XXXXXXXX
				 *          XXXX1101					longueur sur 56 bits
				 * XXXX1111 XXXXXXXX XXXXXXXX XXXXXXXX
				 *          XXXXXXXX XXXXXXXX XXXXXXXX
				 *          XXXXXXXX XXXX1111			longueur sur 64 bits
				 */

				if ((position_finale = ftell((*descripteur).descripteur_c))
						== -1)
				{
					liberation(s_etat_processus, s_objet_argument);

					(*s_etat_processus).erreur_systeme = d_es_erreur_fichier;
					return;
				}

				// Lecture du premier octet. Le pointeur de lecture se
				// trouve aprs l'opration  sa position initiale.

				if (position_finale == 0)
				{
					liberation(s_etat_processus, s_objet_argument);

					(*s_etat_processus).erreur_execution =
							d_ex_debut_de_fichier_atteint;
					return;
				}

				if (fseek((*descripteur).descripteur_c,
						((long) position_finale) - 1, SEEK_SET) != 0)
				{
					liberation(s_etat_processus, s_objet_argument);

					(*s_etat_processus).erreur_systeme = d_es_erreur_fichier;
					return;
				}

				if (fread(tampon, (size_t) sizeof(unsigned char), 1,
						(*descripteur).descripteur_c) != 1)
				{
					liberation(s_etat_processus, s_objet_argument);

					(*s_etat_processus).erreur_systeme = d_es_erreur_fichier;
					return;
				}

				if ((tampon[0] & 0x01) == 0)
				{
					// Longueur sur sept bits
					saut = tampon[0] >> 1;
				}
				else
				{
					// Longueurs suprieures
					nombre_octets = 2 + ((tampon[0] >> 1) & 0x07);

					if ((position_finale - nombre_octets) < 0)
					{
						liberation(s_etat_processus, s_objet_argument);

						(*s_etat_processus).erreur_systeme = d_ex_syntaxe;
						return;
					}

					if (fseek((*descripteur).descripteur_c,
							((long) (position_finale - nombre_octets)),
							SEEK_SET) != 0)
					{
						liberation(s_etat_processus, s_objet_argument);

						(*s_etat_processus).erreur_systeme =
								d_es_erreur_fichier;
						return;
					}

					if (fread(tampon, (size_t) sizeof(unsigned char),
							(size_t) nombre_octets,
							(*descripteur).descripteur_c)
							!= (size_t) nombre_octets)
					{
						liberation(s_etat_processus, s_objet_argument);

						(*s_etat_processus).erreur_systeme =
								d_es_erreur_fichier;
						return;
					}

					// Rcupration du LSB

					saut = (tampon[0] & 0xF0)
							| ((tampon[nombre_octets - 1] & 0x0F) >> 4);

					// Autres octets

					for(i = 1; i < (nombre_octets - 1); i++)
					{
						saut |= ((integer8) tampon[i]) <<
								(((nombre_octets - 1) - i) * 8);
					}
				}

				if (position_finale - saut >= 0)
				{
					if (fseek((*descripteur).descripteur_c,
							(long) (position_finale - saut), SEEK_SET) != 0)
					{
						liberation(s_etat_processus, s_objet_argument);

						(*s_etat_processus).erreur_systeme =
								d_es_erreur_fichier;
						return;
					}
				}
				else
				{
					liberation(s_etat_processus, s_objet_argument);

					(*s_etat_processus).erreur_execution =
							d_ex_debut_de_fichier_atteint;
					return;
				}
			}
		}
		else
		{
			liberation(s_etat_processus, s_objet_argument);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_fichier;
			return;
		}
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	return;
}


/*
================================================================================
  Fonction 'bessel'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_bessel(struct_processus *s_etat_processus)
{
	logical1					creation_expression;

	struct_liste_chainee		*l_element_atome;
	struct_liste_chainee		*l_element_courant;
	struct_liste_chainee		*l_element_precedent;

	struct_objet				*s_copie_argument_1;
	struct_objet				*s_copie_argument_2;
	struct_objet				*s_copie_argument_3;
	struct_objet				*s_objet_argument_1;
	struct_objet				*s_objet_argument_2;
	struct_objet				*s_objet_argument_3;
	struct_objet				*s_objet_resultat;

	unsigned long				i;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  BESSEL ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(fonctions de Bessel)\n\n");
		}
		else
		{
			printf("(Bessel functions)\n\n");
		}

		printf("    3: \"J\", \"Y\", \"I\", \"K\", \"j\", \"y\", "
				"\"i\", \"k\"\n");
		printf("    2: %s, %s\n", d_INT, d_REL);
		printf("    1: %s, %s\n", d_INT, d_REL);
		printf("->  1: %s\n\n", d_REL);

		printf("    3: \"J\", \"Y\", \"I\", \"K\", \"j\", \"y\", "
				"\"i\", \"k\"\n");
		printf("    2: %s, %s\n", d_INT, d_REL);
		printf("    1: %s, %s\n", d_NOM, d_ALG);
		printf("->  1: %s\n\n", d_ALG);

		printf("    3: \"J\", \"Y\", \"I\", \"K\", \"j\", \"y\", "
				"\"i\", \"k\"\n");
		printf("    2: %s, %s\n", d_INT, d_REL);
		printf("    1: %s\n", d_RPN);
		printf("->  1: %s\n", d_RPN);
		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = 3;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 3) == d_erreur)
		{
			return;
		}
	}

	/*
	 * Jn	fonction cylindrique rgulire
	 * Yn	fonction cylindrique irrgulire
	 * In	fonction cylindrique rgulire modifie
	 * Kn	fonction cylindrique irrgulire modifie
	 * jn	fonction sphrique rgulire
	 * yn	fonction sphrique irrgulire
	 * in	fonction sphrique rgulire modifie
	 * kn	fonction sphrique irrgulire modifie
	 *
	 * Attention : Ordre fractionnaire uniquement pour les
	 * fonctions cylindriques
	 */

	creation_expression = d_faux;

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_3) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*s_objet_argument_3).type == CHN)
	{
		if ((strcmp((unsigned char *) (*s_objet_argument_3).objet, "J") == 0) ||
				(strcmp((unsigned char *) (*s_objet_argument_3).objet, "Y")
				== 0) || (strcmp((unsigned char *) (*s_objet_argument_3).objet,
				"I") == 0) || (strcmp((unsigned char *) (*s_objet_argument_3)
				.objet, "K") == 0) || (strcmp((unsigned char *)
				(*s_objet_argument_3).objet, "j") == 0) || (strcmp(
				(unsigned char *) (*s_objet_argument_3).objet, "y") == 0) ||
				(strcmp((unsigned char *) (*s_objet_argument_3).objet, "i") ==
				0) || (strcmp((unsigned char *) (*s_objet_argument_3).objet,
				"k") == 0))
		{
			if ((*s_objet_argument_2).type == INT)
			{
				if ((*s_objet_argument_1).type == INT)
				{
					if ((s_objet_resultat = allocation(s_etat_processus, REL))
							== NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					switch((*((unsigned char *) (*s_objet_argument_3).objet)))
					{
						case 'J' :
						{
							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Jn((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'Y' :
						{
							if ((*((integer8 *) (*s_objet_argument_1).objet))
									<= 0)
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Yn((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'I' :
						{
							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_In((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'K' :
						{
							if ((*((integer8 *) (*s_objet_argument_1).objet))
									<= 0)
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
									   	s_objet_argument_2);
								liberation(s_etat_processus,
									   	s_objet_argument_3);
								liberation(s_etat_processus,
									   	s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Kn((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'j' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									< 0) || ((*((integer8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
									   	s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_jl((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'y' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									<= 0) || ((*((integer8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
									   	s_objet_argument_3);
								liberation(s_etat_processus,
									   	s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_yl((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'i' :
						{
							if ((*((integer8 *) (*s_objet_argument_2).objet))
									< 0)
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
									   	s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									exp(fabs((double) (*((integer8 *)
									(*s_objet_argument_1).objet)))) *
									gsl_sf_bessel_il_scaled(
									(int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'k' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									<= 0) || ((*((integer8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
									   	s_objet_argument_2);
								liberation(s_etat_processus,
									   	s_objet_argument_3);
								liberation(s_etat_processus,
									   	s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									exp(fabs((double) (*((integer8 *)
									(*s_objet_argument_1).objet)))) *
									gsl_sf_bessel_kl_scaled(
									(int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}
					}
				}
				else if ((*s_objet_argument_1).type == REL)
				{
					if ((s_objet_resultat = allocation(s_etat_processus, REL))
							== NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					switch((*((unsigned char *) (*s_objet_argument_3).objet)))
					{
						case 'J' :
						{
							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Jn((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'Y' :
						{
							if ((*((real8 *) (*s_objet_argument_1).objet))
									<= 0)
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
									   	s_objet_argument_2);
								liberation(s_etat_processus,
									   	s_objet_argument_3);
								liberation(s_etat_processus,
									   	s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Yn((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'I' :
						{
							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_In((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'K' :
						{
							if ((*((real8 *) (*s_objet_argument_1).objet))
									<= 0)
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
									   	s_objet_argument_2);
								liberation(s_etat_processus,
									   	s_objet_argument_3);
								liberation(s_etat_processus,
									   	s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Kn((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'j' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									< 0) || ((*((integer8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_jl((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'y' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									<= 0) || ((*((integer8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_yl((int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'i' :
						{
							if ((*((integer8 *) (*s_objet_argument_2).objet))
									< 0)
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									exp(fabs((double) (*((real8 *)
									(*s_objet_argument_1).objet)))) *
									gsl_sf_bessel_il_scaled(
									(int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'k' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									<= 0) || ((*((integer8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									exp(fabs((double) (*((real8 *)
									(*s_objet_argument_1).objet)))) *
									gsl_sf_bessel_kl_scaled(
									(int) ((*((integer8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}
					}
				}
				else if (((*s_objet_argument_1).type == NOM) ||
						((*s_objet_argument_1).type == RPN) ||
						((*s_objet_argument_1).type == ALG))
				{
					creation_expression = d_vrai;
				}
				else
				{
					liberation(s_etat_processus, s_objet_argument_1);
					liberation(s_etat_processus, s_objet_argument_2);
					liberation(s_etat_processus, s_objet_argument_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else if ((*s_objet_argument_2).type == REL)
			{
				if ((*s_objet_argument_1).type == INT)
				{
					if ((s_objet_resultat = allocation(s_etat_processus, REL))
							== NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					switch((*((unsigned char *) (*s_objet_argument_3).objet)))
					{
						case 'J' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									< 0) || ((*((real8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Jnu((double) ((*((real8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));

							break;
						}

						case 'Y' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									<= 0) || ((*((real8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Ynu((double) ((*((real8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'I' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									< 0) || ((*((real8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Inu((double) ((*((real8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'K' :
						{
							if (((*((integer8 *) (*s_objet_argument_1).objet))
									<= 0) || ((*((real8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Knu((double) ((*((real8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((integer8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						default :
						{
							(*s_etat_processus).exception =
									d_ep_resultat_indefini;

							liberation(s_etat_processus, s_objet_argument_1);
							liberation(s_etat_processus, s_objet_argument_2);
							liberation(s_etat_processus, s_objet_argument_3);
							liberation(s_etat_processus, s_objet_resultat);

							return;
							break;
						}
					}
				}
				else if ((*s_objet_argument_1).type == REL)
				{
					if ((s_objet_resultat = allocation(s_etat_processus, REL))
							== NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					switch((*((unsigned char *) (*s_objet_argument_3).objet)))
					{
						case 'J' :
						{
							if (((*((real8 *) (*s_objet_argument_1).objet))
									< 0) || ((*((real8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Jnu((double) ((*((real8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'Y' :
						{
							if (((*((real8 *) (*s_objet_argument_1).objet))
									<= 0) || ((*((real8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Yn((int) ((*((real8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'I' :
						{
							if (((*((real8 *) (*s_objet_argument_1).objet))
									< 0) || ((*((real8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_In((int) ((*((real8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						case 'K' :
						{
							if (((*((real8 *) (*s_objet_argument_1).objet))
									<= 0) || ((*((real8 *)
									(*s_objet_argument_2).objet)) < 0))
							{
								(*s_etat_processus).exception =
										d_ep_resultat_indefini;

								liberation(s_etat_processus,
										s_objet_argument_1);
								liberation(s_etat_processus,
										s_objet_argument_2);
								liberation(s_etat_processus,
										s_objet_argument_3);
								liberation(s_etat_processus,
										s_objet_resultat);

								return;
							}

							(*((real8 *) (*s_objet_resultat).objet)) =
									gsl_sf_bessel_Kn((int) ((*((real8 *)
									(*s_objet_argument_2).objet))),
									(double) ((*((real8 *)
									(*s_objet_argument_1).objet))));
							break;
						}

						default :
						{
							(*s_etat_processus).exception =
									d_ep_resultat_indefini;

							liberation(s_etat_processus, s_objet_argument_1);
							liberation(s_etat_processus, s_objet_argument_2);
							liberation(s_etat_processus, s_objet_argument_3);
							liberation(s_etat_processus, s_objet_resultat);

							return;
							break;
						}
					}
				}
				else
				{
					liberation(s_etat_processus, s_objet_argument_1);
					liberation(s_etat_processus, s_objet_argument_2);
					liberation(s_etat_processus, s_objet_argument_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else if (((*s_objet_argument_2).type == NOM) ||
					((*s_objet_argument_2).type == RPN) ||
					((*s_objet_argument_2).type == ALG))
			{
				creation_expression = d_vrai;
			}
			else
			{
				liberation(s_etat_processus, s_objet_argument_1);
				liberation(s_etat_processus, s_objet_argument_2);
				liberation(s_etat_processus, s_objet_argument_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);
			liberation(s_etat_processus, s_objet_argument_3);

			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
			return;
		}
	}
	else if (((*s_objet_argument_3).type == NOM) ||
			((*s_objet_argument_3).type == RPN) ||
			((*s_objet_argument_3).type == ALG))
	{
		creation_expression = d_vrai;
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);
		liberation(s_etat_processus, s_objet_argument_3);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	if (creation_expression == d_vrai)
	{
		if ((s_copie_argument_1 = copie_objet(s_etat_processus,
				s_objet_argument_1, 'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((s_copie_argument_2 = copie_objet(s_etat_processus,
				s_objet_argument_2, 'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((s_copie_argument_3 = copie_objet(s_etat_processus,
				s_objet_argument_3, 'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_copie_argument_1).type == RPN) ||
				((*s_copie_argument_2).type == RPN) ||
				((*s_copie_argument_3).type == RPN))
		{
			if ((s_objet_resultat = allocation(s_etat_processus, RPN))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}
		}
		else
		{
			if ((s_objet_resultat = allocation(s_etat_processus, ALG))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}
		}

		if (((*s_objet_resultat).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*s_objet_resultat).objet;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_superieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "<<");

		if (((*s_copie_argument_3).type == ALG) ||
				((*s_copie_argument_3).type == RPN))
		{

			l_element_atome = (struct_liste_chainee *)
					(*s_copie_argument_3).objet;

			i = 0;

			while(l_element_atome != NULL)
			{
				i++;
				l_element_atome = (*l_element_atome).suivant;
			}

			if (i < 3)
			{
				if (((*l_element_courant).suivant =
						allocation_maillon(s_etat_processus)) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				l_element_courant = (*l_element_courant).suivant;
				(*l_element_courant).donnee = s_copie_argument_3;
			}
			else
			{
				(*l_element_courant).suivant = (*((struct_liste_chainee *)
						(*s_copie_argument_3).objet)).suivant;

				l_element_precedent = NULL;
				l_element_courant = (*l_element_courant).suivant;

				liberation(s_etat_processus,
						(*((struct_liste_chainee *) (*s_copie_argument_3)
						.objet)).donnee);
				free((*s_copie_argument_3).objet);
				free(s_copie_argument_3);

				while((*l_element_courant).suivant != NULL)
				{
					l_element_precedent = l_element_courant;
					l_element_courant = (*l_element_courant).suivant;
				}

				liberation(s_etat_processus, (*l_element_courant).donnee);
				free(l_element_courant);

				l_element_courant = l_element_precedent;
			}
		}
		else
		{
			if (((*l_element_courant).suivant =
					allocation_maillon(s_etat_processus)) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			l_element_courant = (*l_element_courant).suivant;
			(*l_element_courant).donnee = s_copie_argument_3;
		}

		if (((*s_copie_argument_2).type == ALG) ||
				((*s_copie_argument_2).type == RPN))
		{
			l_element_atome = (struct_liste_chainee *)
					(*s_copie_argument_2).objet;

			i = 0;

			while(l_element_atome != NULL)
			{
				i++;
				l_element_atome = (*l_element_atome).suivant;
			}

			if (i < 3)
			{
				if (((*l_element_courant).suivant =
						allocation_maillon(s_etat_processus)) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				l_element_courant = (*l_element_courant).suivant;
				(*l_element_courant).donnee = s_copie_argument_2;
			}
			else
			{
				(*l_element_courant).suivant = (*((struct_liste_chainee *)
						(*s_copie_argument_2).objet)).suivant;

				l_element_courant = (*l_element_courant).suivant;
				l_element_precedent = NULL;

				liberation(s_etat_processus,
						(*((struct_liste_chainee *) (*s_copie_argument_2)
						.objet)).donnee);
				free((*s_copie_argument_2).objet);
				free(s_copie_argument_2);

				while((*l_element_courant).suivant != NULL)
				{
					l_element_precedent = l_element_courant;
					l_element_courant = (*l_element_courant).suivant;
				}

				liberation(s_etat_processus, (*l_element_courant).donnee);
				free(l_element_courant);

				l_element_courant = l_element_precedent;
			}
		}
		else
		{
			if (((*l_element_courant).suivant =
					allocation_maillon(s_etat_processus)) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			l_element_courant = (*l_element_courant).suivant;
			(*l_element_courant).donnee = s_copie_argument_2;
		}

		if (((*s_copie_argument_1).type == ALG) ||
				((*s_copie_argument_1).type == RPN))
		{
			l_element_atome = (struct_liste_chainee *)
					(*s_copie_argument_1).objet;

			i = 0;

			while(l_element_atome != NULL)
			{
				i++;
				l_element_atome = (*l_element_atome).suivant;
			}

			if (i < 3)
			{
				if (((*l_element_courant).suivant =
						allocation_maillon(s_etat_processus)) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				l_element_courant = (*l_element_courant).suivant;
				(*l_element_courant).donnee = s_copie_argument_1;
			}
			else
			{
				(*l_element_courant).suivant = (*((struct_liste_chainee *)
						(*s_copie_argument_1).objet)).suivant;

				l_element_courant = (*l_element_courant).suivant;
				l_element_precedent = NULL;

				liberation(s_etat_processus,
						(*((struct_liste_chainee *) (*s_copie_argument_1)
						.objet)).donnee);
				free((*s_copie_argument_1).objet);
				free(s_copie_argument_1);

				while((*l_element_courant).suivant != NULL)
				{
					l_element_precedent = l_element_courant;
					l_element_courant = (*l_element_courant).suivant;
				}

				liberation(s_etat_processus, (*l_element_courant).donnee);
				free(l_element_courant);

				l_element_courant = l_element_precedent;
			}
		}
		else
		{
			if (((*l_element_courant).suivant =
					allocation_maillon(s_etat_processus)) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			l_element_courant = (*l_element_courant).suivant;
			(*l_element_courant).donnee = s_copie_argument_1;
		}

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 3;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_bessel;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(7 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "BESSEL");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee = (struct_objet *)
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_inferieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, ">>");

		(*l_element_courant).suivant = NULL;
	}

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);
	liberation(s_etat_processus, s_objet_argument_3);

	return;
}


/*
================================================================================
  Fonction 'backtrace'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_backtrace(struct_processus *s_etat_processus)
{
	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  BACKTRACE ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(affichage de la pile systme)\n\n");
			printf("  Aucun argument\n");
		}
		else
		{
			printf("(print system stack)\n\n");
			printf("  No argument\n");
		}

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 0) == d_erreur)
		{
			return;
		}
	}

	trace(s_etat_processus, stdout);

	return;
}

// vim: ts=4
