/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'd->r'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_d_vers_r(struct_processus *s_etat_processus)
{
	struct_liste_chainee			*l_element_courant;
	struct_liste_chainee			*l_element_precedent;

	struct_objet					*s_copie_argument;
	struct_objet					*s_objet_argument;
	struct_objet					*s_objet_resultat;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  D->R ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(degrs vers radians)\n\n");
		}
		else
		{
			printf("(degrees to radians)\n\n");
		}

		printf("    1: %s, %s\n", d_INT, d_REL);
		printf("->  1: %s\n\n", d_REL);

		printf("    1: %s, %s\n", d_NOM, d_ALG);
		printf("->  1: %s\n\n", d_ALG);

		printf("    1: %s\n", d_RPN);
		printf("->  1: %s\n", d_RPN);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Conversion d'un entier ou d'un rel
--------------------------------------------------------------------------------
*/

	if (((*s_objet_argument).type == INT) ||
			((*s_objet_argument).type == REL))
	{
		if ((s_objet_resultat = allocation(s_etat_processus, REL))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((*s_objet_argument).type == INT)
		{
			(*((real8 *) (*s_objet_resultat).objet)) =
					(real8) (*((integer8 *) (*s_objet_argument).objet));
		}
		else
		{
			(*((real8 *) (*s_objet_resultat).objet)) =
					(*((real8 *) (*s_objet_argument).objet));
		}

		conversion_degres_vers_radians((real8 *) (*s_objet_resultat).objet);
	}

/*
--------------------------------------------------------------------------------
  Conversion d'un nom
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == NOM)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, ALG))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_resultat).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*s_objet_resultat).objet;

		if (((*l_element_courant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_superieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "<<");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;
		(*l_element_courant).donnee = s_objet_argument;

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 1;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_d_vers_r;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(5 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "D->R");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_inferieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, ">>");

		(*l_element_courant).suivant = NULL;
		s_objet_argument = NULL;
	}

/*
--------------------------------------------------------------------------------
  Conversion d'une expression
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument).type == ALG) ||
			((*s_objet_argument).type == RPN))
	{
		if ((s_copie_argument = copie_objet(s_etat_processus,
				s_objet_argument, 'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_copie_argument).objet;
		l_element_precedent = l_element_courant;

		while((*l_element_courant).suivant != NULL)
		{
			l_element_precedent = l_element_courant;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (((*l_element_precedent).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*(*l_element_precedent).suivant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).nombre_arguments = 1;
		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).fonction = instruction_d_vers_r;

		if (((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction =
				malloc(5 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction, "D->R");

		(*(*l_element_precedent).suivant).suivant = l_element_courant;

		s_objet_resultat = s_copie_argument;
	}

/*
--------------------------------------------------------------------------------
  Ralisation impossible de la fonction R->D
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'det'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_det(struct_processus *s_etat_processus)
{
	struct_objet				*s_objet_argument;
	struct_objet				*s_objet_resultat;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  DET ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(dterminant)\n\n");
		}
		else
		{
			printf("(determinant)\n\n");
		}

		printf("    1: %s\n", d_MIN);
		printf("->  1: %s, %s\n\n", d_INT, d_REL);

		printf("    1: %s\n", d_MRL);
		printf("->  1: %s\n\n", d_REL);

		printf("    1: %s\n", d_MCX);
		printf("->  1: %s\n", d_CPL);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  L'argument est une matrice carre
--------------------------------------------------------------------------------
*/

	if (((*s_objet_argument).type == MIN) ||
			((*s_objet_argument).type == MRL))
	{
		if ((*((struct_matrice *) (*s_objet_argument).objet)).nombre_lignes !=
				(*((struct_matrice *) (*s_objet_argument).objet))
				.nombre_colonnes)
		{
			liberation(s_etat_processus, s_objet_argument);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, REL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		determinant(s_etat_processus, (struct_matrice *)
				(*s_objet_argument).objet, (*s_objet_resultat).objet);

		if ((*s_etat_processus).erreur_systeme != d_es)
		{
			return;
		}

		if (((*s_etat_processus).exception != d_ep) ||
				((*s_etat_processus).erreur_execution != d_ex))
		{
			liberation(s_etat_processus, s_objet_resultat);
			liberation(s_etat_processus, s_objet_argument);
			return;
		}
	}
	else if ((*s_objet_argument).type == MCX)
	{
		if ((*((struct_matrice *) (*s_objet_argument).objet)).nombre_lignes !=
				(*((struct_matrice *) (*s_objet_argument).objet))
				.nombre_colonnes)
		{
			liberation(s_etat_processus, s_objet_argument);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		determinant(s_etat_processus, (struct_matrice *)
				(*s_objet_argument).objet,
				((complex16 *) (*s_objet_resultat).objet));

		if ((*s_etat_processus).erreur_systeme != d_es)
		{
			return;
		}

		if (((*s_etat_processus).exception != d_ep) ||
				((*s_etat_processus).erreur_execution != d_ex))
		{
			liberation(s_etat_processus, s_objet_resultat);
			liberation(s_etat_processus, s_objet_argument);
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Type incompatible avec la fonction dterminant
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'dot'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_dot(struct_processus *s_etat_processus)
{
	integer8						cumul;
	integer8						tampon;

	logical1						depassement;
	logical1						erreur_memoire;

	struct_objet					*s_objet_argument_1;
	struct_objet					*s_objet_argument_2;
	struct_objet					*s_objet_resultat;

	integer8						i;

	void							*accumulateur;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  DOT ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(produit scalaire)\n\n");
		}
		else
		{
			printf("(scalar product)\n\n");
		}

		printf("    2: %s\n", d_VIN);
		printf("    1: %s\n", d_VIN);
		printf("->  1: %s, %s\n\n", d_INT, d_REL);

		printf("    2: %s, %s\n", d_VIN, d_VRL);
		printf("    1: %s, %s\n", d_VIN, d_VRL);
		printf("->  1: %s\n\n", d_REL);

		printf("    2: %s, %s, %s\n", d_VIN, d_VRL, d_VCX);
		printf("    1: %s, %s, %s\n", d_VIN, d_VRL, d_VCX);
		printf("->  1: %s\n\n", d_CPL);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 2) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Rsultat entier
--------------------------------------------------------------------------------
*/

	if (((*s_objet_argument_1).type == VIN) &&
			((*s_objet_argument_2).type == VIN))
	{
		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, INT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*s_objet_resultat).objet)) = 0;
		depassement = d_faux;

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			if (depassement_multiplication(&(((integer8 *) (*((struct_vecteur *)
					(*s_objet_argument_1).objet)).tableau)[i]),
					&(((integer8 *) (*((struct_vecteur *) (*s_objet_argument_2)
					.objet)).tableau)[i]), &tampon) == d_erreur)
			{
				depassement = d_vrai;
				break;
			}

			if (depassement_addition((integer8 *) (*s_objet_resultat).objet,
					&tampon, &cumul) == d_erreur)
			{
				depassement = d_vrai;
				break;
			}

			(*((integer8 *) (*s_objet_resultat).objet)) = cumul;
		}

		if (depassement == d_vrai)
		{
			free((*s_objet_resultat).objet);
			(*s_objet_resultat).type = REL;

			if (((*s_objet_resultat).objet = malloc(sizeof(real8))) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
					(*s_objet_argument_1).objet)).taille) * sizeof(real8)))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
					.taille; i++)
			{
				((real8 *) accumulateur)[i] = (real8) ((integer8 *)
						(*((struct_vecteur *) (*s_objet_argument_1)
						.objet)).tableau)[i] * (real8) ((integer8 *)
						(*((struct_vecteur *) (*s_objet_argument_2).objet))
						.tableau)[i];
			}

			(*((real8 *) (*s_objet_resultat).objet)) = sommation_vecteur_reel(
					accumulateur, &((*((struct_vecteur *) (*s_objet_argument_1)
					.objet)).taille), &erreur_memoire);

			if (erreur_memoire == d_vrai)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			free(accumulateur);
		}
	}

/*
--------------------------------------------------------------------------------
  Rsultat rel
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument_1).type == VIN) &&
			((*s_objet_argument_2).type == VRL))
	{
		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, REL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
				(*s_objet_argument_1).objet)).taille) * sizeof(real8))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			((real8 *) accumulateur)[i] = ((real8)
					((integer8 *) (*((struct_vecteur *) (*s_objet_argument_1)
					.objet)).tableau)[i]) * ((real8 *) (*((struct_vecteur *)
					(*s_objet_argument_2).objet)).tableau)[i];
		}

		(*((real8 *) (*s_objet_resultat).objet)) = sommation_vecteur_reel(
				accumulateur, &((*((struct_vecteur *) (*s_objet_argument_1)
				.objet)).taille), &erreur_memoire);

		if (erreur_memoire == d_vrai)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		free(accumulateur);
	}
	else if (((*s_objet_argument_1).type == VRL) &&
			((*s_objet_argument_2).type == VIN))
	{
		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, REL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
				(*s_objet_argument_1).objet)).taille) * sizeof(real8))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			((real8 *) accumulateur)[i] =
					((real8 *) (*((struct_vecteur *) (*s_objet_argument_1)
					.objet)).tableau)[i] * ((real8) ((integer8 *)
					(*((struct_vecteur *)(*s_objet_argument_2).objet))
					.tableau)[i]);
		}

		(*((real8 *) (*s_objet_resultat).objet)) = sommation_vecteur_reel(
				accumulateur, &((*((struct_vecteur *) (*s_objet_argument_1)
				.objet)).taille), &erreur_memoire);

		if (erreur_memoire == d_vrai)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		free(accumulateur);
	}
	else if (((*s_objet_argument_1).type == VRL) &&
			((*s_objet_argument_2).type == VRL))
	{
		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, REL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
				(*s_objet_argument_1).objet)).taille) * sizeof(real8))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			((real8 *) accumulateur)[i] =
					((real8 *) (*((struct_vecteur *) (*s_objet_argument_1)
					.objet)).tableau)[i] * ((real8 *) (*((struct_vecteur *)
					(*s_objet_argument_2).objet)).tableau)[i];
		}

		(*((real8 *) (*s_objet_resultat).objet)) = sommation_vecteur_reel(
				accumulateur, &((*((struct_vecteur *) (*s_objet_argument_1)
				.objet)).taille), &erreur_memoire);

		if (erreur_memoire == d_vrai)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		free(accumulateur);
	}

/*
--------------------------------------------------------------------------------
  Rsultat complexe
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument_1).type == VCX) &&
			((*s_objet_argument_2).type == VIN))
	{
		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
				(*s_objet_argument_1).objet)).taille) * sizeof(complex16)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			f77multiplicationci_(&(((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_1).objet)).tableau)[i]),
					&(((integer8 *) (*((struct_vecteur *) (*s_objet_argument_2)
					.objet)).tableau)[i]), &(((complex16 *) accumulateur)[i]));
		}

		(*((complex16 *) (*s_objet_resultat).objet)) =
				sommation_vecteur_complexe(accumulateur,
				&((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille),
				&erreur_memoire);

		if (erreur_memoire == d_vrai)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		free(accumulateur);
	}
	else if (((*s_objet_argument_1).type == VCX) &&
			((*s_objet_argument_2).type == VRL))
	{
		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
				(*s_objet_argument_1).objet)).taille) * sizeof(complex16)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			f77multiplicationcr_(&(((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_1).objet)).tableau)[i]),
					&(((real8 *) (*((struct_vecteur *) (*s_objet_argument_2)
					.objet)).tableau)[i]), &(((complex16 *) accumulateur)[i]));
		}

		(*((complex16 *) (*s_objet_resultat).objet)) =
				sommation_vecteur_complexe(accumulateur,
				&((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille),
				&erreur_memoire);

		if (erreur_memoire == d_vrai)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		free(accumulateur);
	}
	else if (((*s_objet_argument_1).type == VCX) &&
			((*s_objet_argument_2).type == VCX))
	{
		/*
		 * s_argument_2 est conjugu avant d'effectuer le produit
		 */

		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
				(*s_objet_argument_1).objet)).taille) * sizeof(complex16)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			((struct_complexe16 *) (*((struct_vecteur *) (*s_objet_argument_2)
					.objet)).tableau)[i].partie_imaginaire =
					-((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_2).objet)).tableau)[i].partie_imaginaire;

			f77multiplicationcc_(&(((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_2).objet)).tableau)[i]),
					&(((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_1).objet)).tableau)[i]),
					&(((complex16 *) accumulateur)[i]));
		}

		(*((complex16 *) (*s_objet_resultat).objet)) =
				sommation_vecteur_complexe(accumulateur,
				&((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille),
				&erreur_memoire);

		if (erreur_memoire == d_vrai)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		free(accumulateur);
	}
	else if (((*s_objet_argument_1).type == VRL) &&
			((*s_objet_argument_2).type == VCX))
	{
		/*
		 * s_argument_2 est conjugu avant d'effectuer le produit
		 */

		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
				(*s_objet_argument_1).objet)).taille) * sizeof(complex16)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			((struct_complexe16 *) (*((struct_vecteur *) (*s_objet_argument_2)
					.objet)).tableau)[i].partie_imaginaire =
					-((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_2).objet)).tableau)[i].partie_imaginaire;

			f77multiplicationcr_(&(((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_2).objet)).tableau)[i]),
					&(((real8 *) (*((struct_vecteur *) (*s_objet_argument_1)
					.objet)).tableau)[i]), &(((complex16 *) accumulateur)[i]));
		}

		(*((complex16 *) (*s_objet_resultat).objet)) =
				sommation_vecteur_complexe(accumulateur,
				&((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille),
				&erreur_memoire);

		if (erreur_memoire == d_vrai)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		free(accumulateur);
	}
	else if (((*s_objet_argument_1).type == VIN) &&
			((*s_objet_argument_2).type == VCX))
	{
		/*
		 * s_argument_2 est conjugu avant d'effectuer le produit
		 */

		if ((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille !=
				(*((struct_vecteur *) (*s_objet_argument_2).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((accumulateur = malloc(((size_t) (*((struct_vecteur *)
				(*s_objet_argument_1).objet)).taille) * sizeof(complex16)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_argument_1).objet))
				.taille; i++)
		{
			((struct_complexe16 *) (*((struct_vecteur *) (*s_objet_argument_2)
					.objet)).tableau)[i].partie_imaginaire =
					-((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_2).objet)).tableau)[i].partie_imaginaire;

			f77multiplicationci_(&(((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument_2).objet)).tableau)[i]),
					&(((integer8 *) (*((struct_vecteur *) (*s_objet_argument_1)
					.objet)).tableau)[i]), &(((complex16 *) accumulateur)[i]));
		}

		(*((complex16 *) (*s_objet_resultat).objet)) =
				sommation_vecteur_complexe(accumulateur,
				&((*((struct_vecteur *) (*s_objet_argument_1).objet)).taille),
				&erreur_memoire);

		if (erreur_memoire == d_vrai)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		free(accumulateur);
	}

/*
--------------------------------------------------------------------------------
  Types d'objets incompatibles avec le calcul d'un produit scalaire
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}

// vim: ts=4
