/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'jdate'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_jdate(struct_processus *s_etat_processus)
{
	integer8				a;
	integer8				b;
	integer8				c;
	integer8				d;
	integer8				e;
	integer8				m;
	integer8				jour_julien;

	long					nombre_elements;

	real8					heure_julienne;

	struct_liste_chainee	*l_element_courant;

	struct_objet			*s_objet;
	struct_objet			*s_objet_argument;

	struct timeval			horodatage;

	struct tm				stm;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  JDATE ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(conversion d'un jour julien vers un calendrier grgorien"
					" proleptique)\n\n");
		}
		else
		{
			printf("(conversion from julian day to proleptic gregorian"
					" calendar)\n\n");
		}

		printf("    1: %s\n", d_LST);
		printf("->  1: %s\n\n", d_LST);

		if ((*s_etat_processus).langue == 'F')
		{
			printf("  Utilisation :\n\n");
			printf("    { IP FP } JDATE avec IP > 0, 0 <= FP < 1\n\n");
			printf("  Note :\n\n");
			printf("    La conversion n'est assure que si la date est "
					"reprsentable en temps Unix.\n");
		}
		else
		{
			printf("  Usage:\n\n");
			printf("    { IP FP } JDATE with IP > 0, 0 <= FP < 1\n\n");
			printf("  Note :\n\n");
			printf("    Conversion is only done if date can be computed in "
					"Unix time.\n");
		}

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = 1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*s_objet_argument).type == LST)
	{
		l_element_courant = (*s_objet_argument).objet;
		nombre_elements = 0;

		while(l_element_courant != NULL)
		{
			nombre_elements++;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (nombre_elements != 2)
		{
			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;

			liberation(s_etat_processus, s_objet_argument);
			return;
		}

		l_element_courant = (*s_objet_argument).objet;

		if ((*(*l_element_courant).donnee).type == INT)
		{
			jour_julien = (*((integer8 *) (*(*l_element_courant)
					.donnee).objet));

			if (jour_julien < 0)
			{
				(*s_etat_processus).erreur_execution = d_ex_argument_invalide;

				liberation(s_etat_processus, s_objet_argument);
				return;
			}
		}
		else
		{
			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;

			liberation(s_etat_processus, s_objet_argument);
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if ((*(*l_element_courant).donnee).type == INT)
		{
			heure_julienne = (real8) (*((integer8 *) (*(*l_element_courant)
					.donnee).objet));
		}
		else if ((*(*l_element_courant).donnee).type == REL)
		{
			heure_julienne = (*((real8 *) (*(*l_element_courant)
					.donnee).objet));
		}
		else
		{
			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;

			liberation(s_etat_processus, s_objet_argument);
			return;
		}

		if ((heure_julienne < 0) || (heure_julienne >= 1))
		{
			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;

			liberation(s_etat_processus, s_objet_argument);
			return;
		}

		a = jour_julien + 32045;
		b = ((4 * (a + 36524)) / 146097) - 1;
		c = a - ((146097 * b) / 4);
		d = ((4 * (c + 365)) / 1461) - 1;
		e = c - ((1461 * d) / 4);
		m = ((5 * (e - 1)) + 2) / 153;

		stm.tm_mday = (int) (e - (((153 * m) + 2) / 5));
		stm.tm_mon = (int) (m + 3 - ((12 * (m / 10)) + 1));
		stm.tm_year = (int) ((100 * b) + (m / 10) + d - (4800 + 1900));

		heure_julienne *= 24;
		stm.tm_hour = (int) floor(heure_julienne);
		heure_julienne -= stm.tm_hour;
		stm.tm_hour += 12;

		heure_julienne *= 60;
		stm.tm_min = (int) floor(heure_julienne);
		heure_julienne -= stm.tm_min;

		heure_julienne *= 60;
		stm.tm_sec = (int) floor(heure_julienne);
		heure_julienne -= stm.tm_sec;

		horodatage.tv_sec = mktime(&stm);
		horodatage.tv_usec = (suseconds_t) (heure_julienne * 1000000);

		if (horodatage.tv_sec == -1)
		{
			liberation(s_etat_processus, s_objet_argument);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		if ((s_objet = formateur_date(s_etat_processus, &horodatage)) == NULL)
		{
			liberation(s_etat_processus, s_objet_argument);
			return;
		}

		liberation(s_etat_processus, s_objet_argument);
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet) == d_erreur)
	{
		return;
	}

	return;
}

// vim: ts=4
