/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'limit'
================================================================================
  Entrées : pointeur sur une struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : néant
================================================================================
*/

void
instruction_limit(struct_processus *s_etat_processus)
{
	struct_liste_chainee		*l_element_courant;

	struct_objet				*s_objet_argument_1;
	struct_objet				*s_objet_argument_2;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  LIMIT ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(limite d'une fonction)\n\n");
		}
		else
		{
			printf("(function limit)\n\n");
		}

		printf("    2: %s, %s, %s, %s\n", d_INT, d_REL, d_NOM, d_ALG);
		printf("    1: %s\n", d_LST);
		printf("->  1: %s, %s, %s, %s\n\n", d_INT, d_REL, d_NOM, d_ALG);

		if ((*s_etat_processus).langue == 'F')
		{
			printf("  Utilisation :\n\n");
		}
		else
		{
			printf("  Usage:\n\n");
		}

		printf("    'SIN(X)/X' { 'X' 0 } LIMIT\n");
		printf("    '1/X' { 'X' 0 + } LIMIT\n");
		printf("    'SQRT(X+SQRT(X+SQRT(X)))-SQRT(X)' { 'X' '+infinity' } "
				"LIMIT\n");
		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 2) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (((*s_objet_argument_1).type == LST) &&
			(((*s_objet_argument_2).type == NOM) ||
			((*s_objet_argument_2).type == ALG) ||
			((*s_objet_argument_2).type == REL) ||
			((*s_objet_argument_2).type == INT)))
	{
		l_element_courant = (*s_objet_argument_1).objet;

		if (l_element_courant == NULL)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution =
					d_ex_erreur_type_argument;
			return;
		}

		if ((*(*l_element_courant).donnee).type != NOM)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution =
					d_ex_erreur_type_argument;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (l_element_courant == NULL)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution =
					d_ex_erreur_type_argument;
			return;
		}

		if (((*(*l_element_courant).donnee).type != INT) &&
				((*(*l_element_courant).donnee).type != REL) &&
				((*(*l_element_courant).donnee).type != NOM) &&
				((*(*l_element_courant).donnee).type != ALG))
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution =
					d_ex_erreur_type_argument;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (l_element_courant != NULL)
		{
			// S'il y a un troisième argument, il ne peut être
			// que + ou -.

			if ((*(*l_element_courant).donnee).type != FCT)
			{
				liberation(s_etat_processus, s_objet_argument_1);
				liberation(s_etat_processus, s_objet_argument_2);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			if ((strcmp((*((struct_fonction *) (*(*l_element_courant).donnee)
					.objet)).nom_fonction, "-") != 0) &&
					(strcmp((*((struct_fonction *) (*(*l_element_courant)
					.donnee).objet)).nom_fonction, "+") != 0))
			{
				liberation(s_etat_processus, s_objet_argument_1);
				liberation(s_etat_processus, s_objet_argument_2);

				(*s_etat_processus).erreur_execution =
						d_ex_argument_invalide;
				return;
			}

			if ((*l_element_courant).suivant != NULL)
			{
				liberation(s_etat_processus, s_objet_argument_1);
				liberation(s_etat_processus, s_objet_argument_2);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_argument_2) == d_erreur)
		{
			return;
		}

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_argument_1) == d_erreur)
		{
			return;
		}

		interface_cas(s_etat_processus, RPLCAS_LIMITE);
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution =
				d_ex_erreur_type_argument;
		return;
	}

		return;
}
