/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'obsub'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_obsub(struct_processus *s_etat_processus)
{
	integer8						position;

	struct_liste_chainee			*l_element_courant;

	struct_objet					*s_copie_argument_3;
	struct_objet					*s_objet_argument_1;
	struct_objet					*s_objet_argument_2;
	struct_objet					*s_objet_argument_3;

	unsigned char					*registre_definitions_chainees;
	unsigned char					*registre_instruction_courante;

	integer8						position_courante;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  OBSUB ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(substitution d'objet)\n\n");
		}
		else
		{
			printf("(objet substitution)\n\n");
		}

		printf("    3: %s\n", d_RPN);
		printf("    2: %s\n", d_INT);
		printf("    1: %s\n", d_LST);
		printf("->  1: %s\n\n", d_RPN);

		if ((*s_etat_processus).langue == 'F')
		{
			printf("  Utilisation :\n\n");
		}
		else
		{
			printf("  Usage:\n\n");
		}
	
		printf("    << 1 4 + >> 3 { * } OBSUB\n");
		printf("    << << X >> EVAL SIN >> 2 { << X 1 + >> } OBSUB\n");

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 3) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_3) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (((*s_objet_argument_1).type == LST) &&
			((*s_objet_argument_2).type == INT) &&
			((*s_objet_argument_3).type == RPN))
	{
		if ((s_copie_argument_3 = copie_objet(s_etat_processus,
				s_objet_argument_3, 'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		liberation(s_etat_processus, s_objet_argument_3);
		s_objet_argument_3 = s_copie_argument_3;

		position = 0;
		l_element_courant = (*s_objet_argument_1).objet;

		while(l_element_courant != NULL)
		{
			position++;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (position != 1)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);
			liberation(s_etat_processus, s_objet_argument_3);

			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
			return;
		}

		if ((*((integer8 *) (*s_objet_argument_2).objet)) <= 0)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);
			liberation(s_etat_processus, s_objet_argument_3);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		l_element_courant = (*s_objet_argument_3).objet;
		position = 1;

		while(l_element_courant != NULL)
		{
			if (position == (*((integer8 *) (*s_objet_argument_2).objet)))
			{
				/*
				 * Substitution
				 */

				liberation(s_etat_processus, (*l_element_courant).donnee);

				if (((*l_element_courant).donnee =
						copie_objet(s_etat_processus,
						(*((struct_liste_chainee *)
						(*s_objet_argument_1).objet)).donnee, 'O')) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				/*
				 * Analyse de l'objet rsultant de la substitution
				 */

				position_courante = (*s_etat_processus).position_courante;
				registre_definitions_chainees = (*s_etat_processus)
						.definitions_chainees;
				registre_instruction_courante = (*s_etat_processus)
						.instruction_courante;

				if (((*s_etat_processus).definitions_chainees =
						formateur(s_etat_processus, 0, s_objet_argument_3))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				if (analyse_syntaxique(s_etat_processus) == d_erreur)
				{
					free((*s_etat_processus).definitions_chainees);

					(*s_etat_processus).definitions_chainees =
							registre_definitions_chainees;
					(*s_etat_processus).instruction_courante =
							registre_instruction_courante;
					(*s_etat_processus).position_courante =
							position_courante;

					liberation(s_etat_processus, s_objet_argument_1);
					liberation(s_etat_processus, s_objet_argument_2);
					liberation(s_etat_processus, s_objet_argument_3);

					(*s_etat_processus).erreur_execution =
							d_ex_argument_invalide;
					return;
				}

				free((*s_etat_processus).definitions_chainees);

				(*s_etat_processus).definitions_chainees =
						registre_definitions_chainees;
				(*s_etat_processus).instruction_courante =
						registre_instruction_courante;
				(*s_etat_processus).position_courante =
						position_courante;

				break;
			}

			position++;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (l_element_courant == NULL)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);
			liberation(s_etat_processus, s_objet_argument_3);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);
		liberation(s_etat_processus, s_objet_argument_3);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_argument_3) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'obget'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_obget(struct_processus *s_etat_processus)
{
	struct_liste_chainee			*l_element_courant;

	struct_objet					*s_objet_argument_1;
	struct_objet					*s_objet_argument_2;
	struct_objet					*s_objet_resultat;

	integer8						position;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  OBGET ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(extraction d'un objet)\n\n");
		}
		else
		{
			printf("(get objet)\n\n");
		}

		printf("    2: %s\n", d_RPN);
		printf("    1: %s\n", d_INT);
		printf("->  1: %s\n", d_LST);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 2) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (((*s_objet_argument_1).type == INT) &&
			((*s_objet_argument_2).type == RPN))
	{
		if ((*((integer8 *) (*s_objet_argument_1).objet)) <= 0)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		l_element_courant = (*s_objet_argument_2).objet;
		position = 1;

		while(l_element_courant != NULL)
		{
			if (position == (*((integer8 *) (*s_objet_argument_1).objet)))
			{
				/*
				 * Rcupration de l'objet
				 */

				if ((s_objet_resultat = allocation(s_etat_processus, LST))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				if (((*s_objet_resultat).objet =
						allocation_maillon(s_etat_processus)) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				(*((struct_liste_chainee *) (*s_objet_resultat).objet))
						.suivant = NULL;

				if (((*((struct_liste_chainee *) (*s_objet_resultat).objet))
						.donnee = copie_objet(s_etat_processus,
						(*l_element_courant).donnee, 'O')) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				break;
			}

			position++;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (l_element_courant == NULL)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		/*
		 * Vrification de la cohrence de l'lment. Nous ne devons avoir
		 * ni '<<' ni '>>'.
		 */

		if ((*(*((struct_liste_chainee *) (*s_objet_resultat).objet))
				.donnee).type == FCT)
		{
			if ((strcmp((*((struct_fonction *) (*(*((struct_liste_chainee *)
					(*s_objet_resultat).objet)).donnee).objet)).nom_fonction,
					"<<") == 0) || (strcmp((*((struct_fonction *)
					(*(*((struct_liste_chainee *) (*s_objet_resultat).objet))
					.donnee).objet)).nom_fonction, ">>") == 0))
			{
				liberation(s_etat_processus, s_objet_argument_1);
				liberation(s_etat_processus, s_objet_argument_2);
				liberation(s_etat_processus, s_objet_resultat);

				(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
				return;
			}
		}
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}

// vim: ts=4
