/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'rad'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rad(struct_processus *s_etat_processus)
{
	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RAD ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(arguments en radians)\n\n");
			printf("  Aucun argument\n");
		}
		else
		{
			printf("(radians)\n\n");
			printf("  No argument\n");
		}

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	sf(s_etat_processus, 60);

	return;
}


/*
================================================================================
  Fonction 'roll'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_roll(struct_processus *s_etat_processus)
{
	struct_liste_chainee					*l_liste1;
	struct_liste_chainee					*l_liste2;

	struct_objet							*s_objet;

	integer8								i;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  ROLL ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(dfilement d'un objet vers le haut)\n\n");
		}
		else
		{
			printf("(roll up objects on stack)\n\n");
		}

		printf("  n+1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    ...\n");
		printf("    2: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    1: %s\n", d_INT);
		printf("->  n: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    ...\n");
		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*s_objet).type != INT)
	{
		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	if ((*((integer8 *) (*s_objet).objet)) <= 0)
	{

/*
-- Opration absurde autorise sur le calculateur HP-28S -----------------------
*/

		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
		return;
	}

	if ((*((integer8 *) (*s_objet).objet)) > (integer8) (*s_etat_processus)
			.hauteur_pile_operationnelle)
	{
		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*((integer8 *) (*s_objet).objet)) > 1)
	{
		l_liste1 = (*s_etat_processus).l_base_pile;

		for(i = 2; i < (*((integer8 *) (*s_objet).objet)); i++)
		{
			l_liste1 = (*l_liste1).suivant;
		}

		l_liste2 = (*l_liste1).suivant;
		(*l_liste1).suivant = (*l_liste2).suivant;
		(*l_liste2).suivant = (*s_etat_processus).l_base_pile;
		(*s_etat_processus).l_base_pile = l_liste2;
	}

	liberation(s_etat_processus, s_objet);

	return;
}


/*
================================================================================
  Fonction 'rolld'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rolld(struct_processus *s_etat_processus)
{
	struct_liste_chainee						*l_liste1;
	struct_liste_chainee						*l_liste2;

	struct_objet								*s_objet;

	integer8									i;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  ROLLD ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(dfilement d'un objet vers le bas)\n\n");
		}
		else
		{
			printf("(roll down objects on stack)\n\n");
		}

		printf("  n+1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    ...\n");
		printf("    2: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    1: %s\n", d_INT);
		printf("->  n: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    ...\n");
		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*s_objet).type != INT)
	{
		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	if ((*((integer8 *) (*s_objet).objet)) <= 0)
	{

/*
-- Opration absurde autorise sur le calculateur HP-28S -----------------------
*/

		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
		return;
	}

	if ((*((integer8 *) (*s_objet).objet)) > (integer8) (*s_etat_processus)
			.hauteur_pile_operationnelle)
	{
		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*((integer8 *) (*s_objet).objet)) > 1)
	{
		l_liste1 = (*s_etat_processus).l_base_pile;

		for(i = 1; i < (*((integer8 *) (*s_objet).objet)); i++)
		{
			l_liste1 = (*l_liste1).suivant;
		}

		l_liste2 = (*s_etat_processus).l_base_pile;
		(*s_etat_processus).l_base_pile = (*(*s_etat_processus)
				.l_base_pile).suivant;
		(*l_liste2).suivant = (*l_liste1).suivant;
		(*l_liste1).suivant = l_liste2;
	}

	liberation(s_etat_processus, s_objet);

	return;
}


/*
================================================================================
  Fonction 'rot'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rot(struct_processus *s_etat_processus)
{
	struct_liste_chainee				*l_liste1;
	struct_liste_chainee				*l_liste2;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  ROT ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(rotation)\n\n");
		}
		else
		{
			printf("(rotation)\n");
		}

		printf("    3: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    2: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("->  3: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    2: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 3) == d_erreur)
		{
			return;
		}
	}

	if ((*s_etat_processus).hauteur_pile_operationnelle < 3)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	l_liste1 = (*(*s_etat_processus).l_base_pile).suivant;
	l_liste2 = (*l_liste1).suivant;
	(*l_liste1).suivant = (*l_liste2).suivant;
	(*l_liste2).suivant = (*s_etat_processus).l_base_pile;
	(*s_etat_processus).l_base_pile = l_liste2;
	
	return;
}


/*
================================================================================
  Fonction 'repeat'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_repeat(struct_processus *s_etat_processus)
{
	struct_objet					*s_objet;

	logical1						condition;
	logical1						drapeau_fin;
	logical1						execution;

	struct_liste_chainee			*s_registre;

	unsigned char					*instruction_majuscule;
	unsigned char					*tampon;

	integer8						niveau;

	void							(*fonction)();

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  REPEAT ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(structure de contrle)\n\n");
			printf("  Utilisation :\n\n");
		}
		else
		{
			printf("(control statement)\n\n");
			printf("  Usage:\n\n");
		}

		printf("    WHILE\n");
		printf("        (clause)\n");
		printf("    REPEAT\n");
		printf("        (expression 1)\n");
		printf("        EXIT\n");
		printf("        (expression 2)\n");
		printf("    END\n\n");

		printf("    WHILE\n");
		printf("        (clause)\n");
		printf("    REPEAT\n");
		printf("        (expression)\n");
		printf("    END\n");

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet) == d_erreur)
	{
		return;
	}

	if (((*s_objet).type == INT) ||
			((*s_objet).type == REL))
	{
		if ((*s_objet).type == INT)
		{
			condition = ((*((integer8 *) (*s_objet).objet)) == 0)
					? d_faux : d_vrai;
		}
		else
		{
			condition = ((*((real8 *) (*s_objet).objet)) == 0)
					? d_faux : d_vrai;
		}

		if (condition == d_faux)
		{
			niveau = 0;
			(*(*s_etat_processus).l_base_pile_systeme).clause = 'M';
			drapeau_fin = d_faux;

			if ((*s_etat_processus).mode_execution_programme == 'Y')
			{
				tampon = (*s_etat_processus).instruction_courante;

				do
				{
					if (recherche_instruction_suivante(s_etat_processus) !=
							d_absence_erreur)
					{
						liberation(s_etat_processus, s_objet);

						if ((*s_etat_processus).instruction_courante != NULL)
						{
							free((*s_etat_processus).instruction_courante);
						}

						(*s_etat_processus).instruction_courante = tampon;
						(*s_etat_processus).erreur_execution =
								d_ex_erreur_traitement_condition;
						return;
					}

					if ((instruction_majuscule = conversion_majuscule(
							s_etat_processus,
							(*s_etat_processus).instruction_courante)) == NULL)
					{
						liberation(s_etat_processus, s_objet);

						free((*s_etat_processus).instruction_courante);
						(*s_etat_processus).instruction_courante = tampon;
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}


					if (niveau == 0)
					{
						if ((strcmp(instruction_majuscule, "END") == 0)
								|| (strcmp(instruction_majuscule, "ELSE") == 0)
								|| (strcmp(instruction_majuscule, "ELSEIF")
								== 0))
						{
							(*s_etat_processus).position_courante -=
									(integer8) (strlen(
									instruction_majuscule) + 1);
							drapeau_fin = d_vrai;
						}
						else
						{
							drapeau_fin = d_faux;
						}
					}
					else
					{
						drapeau_fin = d_faux;
					}

					if ((strcmp(instruction_majuscule, "CASE") == 0) ||
							(strcmp(instruction_majuscule, "DO") == 0) ||
							(strcmp(instruction_majuscule, "IF") == 0) ||
							(strcmp(instruction_majuscule, "IFERR") == 0) ||
							(strcmp(instruction_majuscule, "SELECT") == 0)
							|| (strcmp(instruction_majuscule, "WHILE")
							== 0))
					{
						niveau++;
					}
					else if (strcmp(instruction_majuscule, "END") == 0)
					{
						niveau--;
					}

					free(instruction_majuscule);
					free((*s_etat_processus).instruction_courante);
				} while(drapeau_fin == d_faux);

				(*s_etat_processus).instruction_courante = tampon;
			}
			else
			{
				/*
				 * Vrification du pointeur de prdiction de saut
				 */

				if (pthread_mutex_lock(&((*(*(*s_etat_processus)
						.expression_courante).donnee).mutex)) != 0)
				{
					(*s_etat_processus).erreur_systeme = d_es_processus;
					return;
				}

				if ((*((struct_fonction *) (*(*(*s_etat_processus)
						.expression_courante).donnee).objet)).prediction_saut
						!= NULL)
				{
					s_registre = (*s_etat_processus).expression_courante;

					(*s_etat_processus).expression_courante =
							(struct_liste_chainee *)
							(*((struct_fonction *) (*(*(*s_etat_processus)
							.expression_courante).donnee).objet))
							.prediction_saut;
					fonction = (*((struct_fonction *)
							(*(*(*s_etat_processus).expression_courante)
							.donnee).objet)).fonction;
					execution = (*((struct_fonction *)
							(*(*s_registre).donnee).objet))
							.prediction_execution;

					if (pthread_mutex_unlock(&((*(*s_registre).donnee).mutex))
							!= 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}

					if (execution == d_vrai)
					{
						fonction(s_etat_processus);
					}
				}
				else
				{
					if (pthread_mutex_unlock(&((*(*(*s_etat_processus)
							.expression_courante).donnee).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}

					s_registre = (*s_etat_processus).expression_courante;
					execution = d_faux;

					do
					{
						if (((*s_etat_processus).expression_courante =
								(*(*s_etat_processus)
								.expression_courante).suivant) == NULL)
						{
							liberation(s_etat_processus, s_objet);

							(*s_etat_processus).erreur_execution =
									d_ex_erreur_traitement_condition;
							return;
						}

						if ((*(*(*s_etat_processus).expression_courante)
								.donnee).type == FCT)
						{
							fonction = (*((struct_fonction *)
									(*(*(*s_etat_processus).expression_courante)
									.donnee).objet)).fonction;

							if (niveau == 0)
							{
								if ((fonction == instruction_end) ||
										(fonction == instruction_else) ||
										(fonction == instruction_elseif))
								{
									fonction(s_etat_processus);
									execution = d_vrai;
									drapeau_fin = d_vrai;
								}
								else
								{
									drapeau_fin = d_faux;
								}
							}
							else
							{
								drapeau_fin = d_faux;
							}

							if ((fonction == instruction_case) ||
									(fonction == instruction_do) ||
									(fonction == instruction_if) ||
									(fonction == instruction_iferr) ||
									(fonction == instruction_select) ||
									(fonction == instruction_while))
							{
								niveau++;
							}
							else if (fonction == instruction_end)
							{
								niveau--;
							}
						}
					} while(drapeau_fin == d_faux);

					if (pthread_mutex_lock(&((*(*(*s_etat_processus)
							.expression_courante).donnee).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}

					(*((struct_fonction *) (*(*s_registre).donnee).objet))
							.prediction_saut = (*s_etat_processus)
							.expression_courante;
					(*((struct_fonction *) (*(*s_registre).donnee).objet))
							.prediction_execution = execution;

					if (pthread_mutex_unlock(&((*(*(*s_etat_processus)
							.expression_courante).donnee).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}
			}
		}
	}
	else
	{
		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet);

	return;
}


/*
================================================================================
  Fonction 'rclf'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rclf(struct_processus *s_etat_processus)
{
	struct_objet						*s_objet_resultat;

	t_8_bits							masque;

	unsigned char						indice_bit;
	unsigned char						indice_bloc;
	unsigned char						indice_drapeau;
	unsigned char						taille_bloc;

	unsigned long						i;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RCLF ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(renvoie les drapeaux d'tat)\n\n");
		}
		else
		{
			printf("(recall flags)\n\n");
		}

		printf("->  1: %s\n", d_BIN);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}
	
	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 0) == d_erreur)
		{
			return;
		}
	}

	if ((s_objet_resultat = allocation(s_etat_processus, BIN)) == NULL)
	{
		(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
		return;
	}

	(*((logical8 *) (*s_objet_resultat).objet)) = 0;
	taille_bloc = sizeof(t_8_bits) * 8;

	for(i = 1; i <= 64; i++)
	{
		indice_drapeau = (unsigned char) (i - 1);
		indice_bloc = indice_drapeau / taille_bloc;
		indice_bit = indice_drapeau % taille_bloc;
		masque = (t_8_bits) (((t_8_bits) 1) << (taille_bloc - indice_bit - 1));

		if (((*s_etat_processus).drapeaux_etat[indice_bloc] & masque) != 0)
		{
			(*((logical8 *) (*s_objet_resultat).objet)) |=
					((logical8) 1) << indice_drapeau;
		}
	}

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'rcl'
================================================================================
  Entres : structure processus
-------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rcl(struct_processus *s_etat_processus)
{
	struct_objet						*s_objet;
	struct_objet						*s_objet_variable;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RCL ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(renvoie le contenu d'une variable globale)\n\n");
		}
		else
		{
			printf("(recall global variable)\n\n");
		}

		printf("    1: %s\n", d_NOM);
		printf("->  1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}
	
	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*s_objet).type != NOM)
	{
		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	if (recherche_variable_globale(s_etat_processus, (*((struct_nom *)
			(*s_objet).objet)).nom) == d_faux)
	{
		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_systeme = d_es;

		if ((*s_etat_processus).erreur_execution == d_ex)
		{
			(*s_etat_processus).erreur_execution = d_ex_variable_non_definie;
		}

		return;
	}

	if ((s_objet_variable = copie_objet(s_etat_processus,
			(*(*s_etat_processus).pointeur_variable_courante).objet, 'P'))
			== NULL)
	{
		(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
		return;
	}

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_variable) == d_erreur)
	{
		return;
	}

	liberation(s_etat_processus, s_objet);

	return;
}


/*
================================================================================
  Fonction 'rand'
================================================================================
  Entres : structure processus
-------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rand(struct_processus *s_etat_processus)
{
	struct_objet				*s_objet;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RAND ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(variable alatoire uniforme)\n\n");
		}
		else
		{
			printf("(uniform random number)\n\n");
		}

		printf("->  1: %s\n", d_REL);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}
	
	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 0) == d_erreur)
		{
			return;
		}
	}

	if ((*s_etat_processus).generateur_aleatoire == NULL)
	{
		initialisation_generateur_aleatoire(s_etat_processus, d_vrai, 0);
	}

	if ((s_objet = allocation(s_etat_processus, REL)) == NULL)
	{
		(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
		return;
	}

	(*((real8 *) (*s_objet).objet)) = gsl_rng_uniform(
			(*s_etat_processus).generateur_aleatoire);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'rdz'
================================================================================
  Entres : structure processus
-------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rdz(struct_processus *s_etat_processus)
{
	struct_objet				*s_objet;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RDZ ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(racine des nombres alatoires)\n\n");
		}
		else
		{
			printf("(random seed)\n\n");
		}

		printf("    1: %s\n", d_INT);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}
	
	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet) == d_erreur)
	{
		return;
	}

	if ((*s_objet).type == INT)
	{
		initialisation_generateur_aleatoire(s_etat_processus, d_faux,
				(*((integer8 *) (*s_objet).objet)));
	}
	else
	{
		liberation(s_etat_processus, s_objet);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet);
}


/*
================================================================================
  Fonction 'rnd'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rnd(struct_processus *s_etat_processus)
{
	struct_objet						*s_objet_argument;

	unsigned char						*instruction_courante;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RND ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(arrondi)\n\n");
		}
		else
		{
			printf("(rounding)\n\n");
		}

		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX);
		printf("->  1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = 1;
		return;
	}
	
	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (((*s_objet_argument).type == INT) ||
			((*s_objet_argument).type == REL) ||
			((*s_objet_argument).type == CPL) ||
			((*s_objet_argument).type == VIN) ||
			((*s_objet_argument).type == VRL) ||
			((*s_objet_argument).type == VCX) ||
			((*s_objet_argument).type == MIN) ||
			((*s_objet_argument).type == MRL) ||
			((*s_objet_argument).type == MCX))
	{
		instruction_courante = (*s_etat_processus).instruction_courante;

		if (((*s_etat_processus).instruction_courante =
				formateur(s_etat_processus, 0, s_objet_argument)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			(*s_etat_processus).instruction_courante = instruction_courante;
			return;
		}

		(*s_etat_processus).type_en_cours = NON;
		recherche_type(s_etat_processus);

		free((*s_etat_processus).instruction_courante);
		(*s_etat_processus).instruction_courante = instruction_courante;

		if ((*s_etat_processus).erreur_systeme != d_es)
		{
			return;
		}

		if ((*s_etat_processus).erreur_execution != d_ex)
		{
			liberation(s_etat_processus, s_objet_argument);
			return;
		}
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	return;
}


/*
================================================================================
  Fonction 'r->c'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_r_vers_c(struct_processus *s_etat_processus)
{
	struct_objet					*s_objet_argument_1;
	struct_objet					*s_objet_argument_2;
	struct_objet					*s_objet_resultat;

	integer8						i;
	integer8						j;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  R->C ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(rel vers complexe)\n\n");
		}
		else
		{
			printf("(real to complex)\n\n");
		}

		printf("    2: %s, %s\n", d_INT, d_REL);
		printf("    1: %s, %s\n", d_INT, d_REL);
		printf("->  1: %s\n\n", d_CPL);

		printf("    2: %s, %s\n", d_VIN, d_VRL);
		printf("    1: %s, %s\n", d_VIN, d_VRL);
		printf("->  1: %s\n\n", d_VCX);

		printf("    2: %s, %s\n", d_MIN, d_MRL);
		printf("    1: %s, %s\n", d_MIN, d_MRL);
		printf("->  1: %s\n", d_MCX);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 2) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Formation d'un complexe  partir de deux rels
--------------------------------------------------------------------------------
*/

	if ((((*s_objet_argument_1).type == INT) ||
			((*s_objet_argument_1).type == REL)) &&
			(((*s_objet_argument_2).type == INT) ||
			((*s_objet_argument_2).type == REL)))
	{
		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((*s_objet_argument_1).type == INT)
		{
			(*((struct_complexe16 *) (*s_objet_resultat).objet))
					.partie_imaginaire = (real8)
					(*((integer8 *) (*s_objet_argument_1).objet));
		}
		else
		{
			(*((struct_complexe16 *) (*s_objet_resultat).objet))
			   		.partie_imaginaire =
					(*((real8 *) (*s_objet_argument_1).objet));
		}

		if ((*s_objet_argument_2).type == INT)
		{
			(*((struct_complexe16 *) (*s_objet_resultat).objet))
					.partie_reelle = (real8)
					(*((integer8 *) (*s_objet_argument_2).objet));
		}
		else
		{
			(*((struct_complexe16 *) (*s_objet_resultat).objet))
			   		.partie_reelle =
					(*((real8 *) (*s_objet_argument_2).objet));
		}
	}

/*
--------------------------------------------------------------------------------
  Formation  partir de deux vecteurs
--------------------------------------------------------------------------------
*/

	else if ((((*s_objet_argument_1).type == VIN) ||
			((*s_objet_argument_1).type == VRL)) &&
			(((*s_objet_argument_2).type == VIN) ||
			((*s_objet_argument_2).type == VRL)))
	{
		if ((*(((struct_vecteur *) (*s_objet_argument_1).objet))).taille !=
				(*(((struct_vecteur *) (*s_objet_argument_2).objet))).taille)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, VCX)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_vecteur *) (*s_objet_resultat).objet)).taille =
				(*((struct_vecteur *) (*s_objet_argument_1).objet)).taille;

		if (((*((struct_vecteur *) (*s_objet_resultat).objet)).tableau =
				malloc(((size_t) (*(((struct_vecteur *) (*s_objet_resultat)
				.objet))).taille) * sizeof(struct_complexe16))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*(((struct_vecteur *) (*s_objet_argument_1).objet)))
				.taille; i++)
		{
			if ((*s_objet_argument_1).type == VIN)
			{
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_resultat).objet)).tableau)[i]
						.partie_imaginaire = (real8) ((integer8 *)
						(*((struct_vecteur *) (*s_objet_argument_1).objet))
						.tableau)[i];
			}
			else
			{
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_resultat).objet)).tableau)[i]
						.partie_imaginaire = ((real8 *)
						(*((struct_vecteur *) (*s_objet_argument_1).objet))
						.tableau)[i];
			}

			if ((*s_objet_argument_2).type == VIN)
			{
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_resultat).objet)).tableau)[i]
						.partie_reelle = (real8) ((integer8 *)
						(*((struct_vecteur *) (*s_objet_argument_2).objet))
						.tableau)[i];
			}
			else
			{
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_resultat).objet)).tableau)[i]
						.partie_reelle = ((real8 *)
						(*((struct_vecteur *) (*s_objet_argument_2).objet))
						.tableau)[i];
			}
		}
	}

/*
--------------------------------------------------------------------------------
  Formation  partir de deux matrices
--------------------------------------------------------------------------------
*/

	else if ((((*s_objet_argument_1).type == MIN) ||
			((*s_objet_argument_1).type == MRL)) &&
			(((*s_objet_argument_2).type == MIN) ||
			((*s_objet_argument_2).type == MRL)))
	{
		if (((*(((struct_matrice *) (*s_objet_argument_1).objet)))
				.nombre_lignes != (*(((struct_matrice *) (*s_objet_argument_2)
				.objet))).nombre_lignes) || ((*(((struct_matrice *)
				(*s_objet_argument_1).objet))).nombre_colonnes !=
				(*(((struct_matrice *) (*s_objet_argument_2).objet)))
				.nombre_lignes))
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, MCX)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_matrice *) (*s_objet_resultat).objet)).nombre_lignes =
				(*((struct_matrice *) (*s_objet_argument_1).objet))
				.nombre_lignes;
		(*((struct_matrice *) (*s_objet_resultat).objet)).nombre_colonnes =
				(*((struct_matrice *) (*s_objet_argument_1).objet))
				.nombre_colonnes;

		if (((*((struct_matrice *) (*s_objet_resultat).objet)).tableau =
				malloc(((size_t) (*(((struct_matrice *) (*s_objet_resultat)
				.objet))).nombre_lignes) * sizeof(struct_complexe16 *)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		for(i = 0; i < (*(((struct_matrice *) (*s_objet_argument_1).objet)))
				.nombre_lignes; i++)
		{
			if ((((struct_complexe16 **) (*((struct_matrice *)
					(*s_objet_resultat).objet)).tableau)[i]	=
					malloc(((size_t) (*((struct_matrice *)
					(*s_objet_resultat).objet)).nombre_colonnes) *
					sizeof(struct_complexe16))) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			for(j = 0; j < (*(((struct_matrice *) (*s_objet_argument_1).objet)))
					.nombre_colonnes; j++)
			{
				if ((*s_objet_argument_1).type == MIN)
				{
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_resultat).objet)).tableau)[i][j]
							.partie_imaginaire = (real8) ((integer8 **)
							(*((struct_matrice *) (*s_objet_argument_1).objet))
							.tableau)[i][j];
				}
				else
				{
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_resultat).objet)).tableau)[i][j]
							.partie_imaginaire = ((real8 **)
							(*((struct_matrice *) (*s_objet_argument_1).objet))
							.tableau)[i][j];
				}

				if ((*s_objet_argument_2).type == MIN)
				{
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_resultat).objet)).tableau)[i][j]
							.partie_reelle = (real8) ((integer8 **)
							(*((struct_matrice *) (*s_objet_argument_2).objet))
							.tableau)[i][j];
				}
				else
				{
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_resultat).objet)).tableau)[i][j]
							.partie_reelle = ((real8 **)
							(*((struct_matrice *) (*s_objet_argument_2).objet))
							.tableau)[i][j];
				}
			}
		}
	}

/*
--------------------------------------------------------------------------------
  Formation impossible
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 're'
================================================================================
  Entres : structure processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_re(struct_processus *s_etat_processus)
{
	struct_liste_chainee			*l_element_courant;
	struct_liste_chainee			*l_element_precedent;

	struct_objet					*s_copie_argument;
	struct_objet					*s_objet_argument;
	struct_objet					*s_objet_resultat;

	integer8						i;
	integer8						j;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RE ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(partie relle)\n\n");
		}
		else
		{
			printf("(real part)\n\n");
		}

		printf("    1: %s, %s\n", d_INT, d_REL);
		printf("->  1: %s\n\n", d_INT);

		printf("    1: %s\n", d_CPL);
		printf("->  1: %s\n\n", d_REL);

		printf("    1: %s, %s\n", d_VIN, d_VRL);
		printf("->  1: %s\n\n", d_VIN);

		printf("    1: %s\n", d_VCX);
		printf("->  1: %s\n\n", d_VRL);

		printf("    1: %s, %s\n", d_MIN, d_MRL);
		printf("->  1: %s\n\n", d_MIN);

		printf("    1: %s\n", d_MCX);
		printf("->  1: %s\n\n", d_MRL);

		printf("    1: %s, %s\n", d_NOM, d_ALG);
		printf("->  1: %s\n\n", d_ALG);

		printf("    1: %s\n", d_RPN);
		printf("->  1: %s\n", d_RPN);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = 1;
		return;
	}
	
	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Partie relle d'un entier ou d'un rel
--------------------------------------------------------------------------------
*/

	if (((*s_objet_argument).type == INT) ||
			((*s_objet_argument).type == REL))
	{
		s_objet_resultat = s_objet_argument;
		s_objet_argument = NULL;
	}

/*
--------------------------------------------------------------------------------
  Partie relle d'un complexe
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == CPL)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, REL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((real8 *) (*s_objet_resultat).objet)) =
				(*((struct_complexe16 *) (*s_objet_argument).objet))
				.partie_reelle;
	}

/*
--------------------------------------------------------------------------------
  Partie relle d'un vecteur
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument).type == VIN) ||
			((*s_objet_argument).type == VRL))
	{
		s_objet_resultat = s_objet_argument;
		s_objet_argument = NULL;
	}
	else if ((*s_objet_argument).type == VCX)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, VRL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*((struct_vecteur *) (*s_objet_resultat).objet)).tableau =
				malloc(((size_t) (*(((struct_vecteur *) (*s_objet_argument)
				.objet))).taille) * sizeof(real8))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_vecteur *) (*s_objet_resultat).objet)).taille =
				(*(((struct_vecteur *) (*s_objet_argument).objet))).taille;

		for(i = 0; i < (*(((struct_vecteur *) (*s_objet_argument).objet)))
				.taille; i++)
		{
			((real8 *) (*((struct_vecteur *) (*s_objet_resultat).objet))
					.tableau)[i] = ((struct_complexe16 *) (*((struct_vecteur *)
					(*s_objet_argument).objet)).tableau)[i].partie_reelle;
		}
	}

/*
--------------------------------------------------------------------------------
  Partie relle d'une matrice
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument).type == MIN) ||
			((*s_objet_argument).type == MRL))
	{
		s_objet_resultat = s_objet_argument;
		s_objet_argument = NULL;
	}
	else if ((*s_objet_argument).type == MCX)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, MRL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*((struct_matrice *) (*s_objet_resultat).objet)).tableau =
				malloc(((size_t) (*(((struct_matrice *) (*s_objet_argument)
				.objet))).nombre_lignes) * sizeof(real8 *))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_matrice *) (*s_objet_resultat).objet)).nombre_lignes =
				(*((struct_matrice *) (*s_objet_argument).objet)).nombre_lignes;
		(*((struct_matrice *) (*s_objet_resultat).objet)).nombre_colonnes =
				(*((struct_matrice *) (*s_objet_argument).objet))
				.nombre_colonnes;

		for(i = 0; i < (*(((struct_matrice *) (*s_objet_argument).objet)))
				.nombre_lignes; i++)
		{
			if ((((real8 **) (*((struct_matrice *) (*s_objet_resultat)
					.objet)).tableau)[i] = malloc(((size_t) 
					(*(((struct_matrice *) (*s_objet_argument).objet)))
					.nombre_colonnes) * sizeof(real8))) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			for(j = 0; j < (*(((struct_matrice *) (*s_objet_argument).objet)))
					.nombre_colonnes; j++)
			{
				((real8 **) (*((struct_matrice *) (*s_objet_resultat).objet))
						.tableau)[i][j] = ((struct_complexe16 **)
						(*((struct_matrice *) (*s_objet_argument).objet))
						.tableau)[i][j].partie_reelle;
			}
		}
	}

/*
--------------------------------------------------------------------------------
  Partie relle d'un nom
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == NOM)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, ALG)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_resultat).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*s_objet_resultat).objet;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_superieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "<<");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;
		(*l_element_courant).donnee = s_objet_argument;

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 1;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_re;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "RE");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_inferieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, ">>");

		(*l_element_courant).suivant = NULL;
		s_objet_argument = NULL;
	}

/*
--------------------------------------------------------------------------------
  Partie relle d'une expression
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument).type == ALG) ||
			((*s_objet_argument).type == RPN))
	{
		if ((s_copie_argument = copie_objet(s_etat_processus,
				s_objet_argument, 'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_copie_argument).objet;
		l_element_precedent = l_element_courant;

		while((*l_element_courant).suivant != NULL)
		{
			l_element_precedent = l_element_courant;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (((*l_element_precedent).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*(*l_element_precedent).suivant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).nombre_arguments = 1;
		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).fonction = instruction_re;

		if (((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction =
				malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction, "RE");

		(*(*l_element_precedent).suivant).suivant = l_element_courant;

		s_objet_resultat = s_copie_argument;
	}

/*
--------------------------------------------------------------------------------
  Ralisation impossible de la fonction partie relle
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'r->p'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_r_vers_p(struct_processus *s_etat_processus)
{
	struct_liste_chainee			*l_element_courant;
	struct_liste_chainee			*l_element_precedent;

	struct_objet					*s_copie_argument;
	struct_objet					*s_objet_argument;
	struct_objet					*s_objet_resultat;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  P->R ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(coordonnes polaires vers cartsiennes)\n\n");
		}
		else
		{
			printf("(polar to cartesian coordinates)\n\n");
		}

		printf("    1: %s, %s, %s\n", d_INT, d_REL, d_CPL);
		printf("->  1: %s\n\n", d_CPL);

		printf("    1: %s, %s\n", d_NOM, d_ALG);
		printf("->  1: %s\n\n", d_ALG);

		printf("    1: %s\n", d_RPN);
		printf("->  1: %s\n", d_RPN);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Conversion d'un entier ou d'un rel
--------------------------------------------------------------------------------
*/

	if (((*s_objet_argument).type == INT) ||
			((*s_objet_argument).type == REL))
	{
		if ((s_objet_resultat = allocation(s_etat_processus, CPL))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((*s_objet_argument).type == INT)
		{
			(*((struct_complexe16 *) (*s_objet_resultat).objet)).partie_reelle
					= (real8) (*((integer8 *) (*s_objet_argument).objet));
		}
		else
		{
			(*((struct_complexe16 *) (*s_objet_resultat).objet)).partie_reelle
					= (*((real8 *) (*s_objet_argument).objet));
		}

		(*((struct_complexe16 *) (*s_objet_resultat).objet)).partie_imaginaire
				= 0;
	}

/*
--------------------------------------------------------------------------------
  Conversion d'un complexe
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == CPL)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, CPL))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		f77absc_(((struct_complexe16 *) (*s_objet_argument).objet),
				&((*((struct_complexe16 *) (*s_objet_resultat).objet))
				.partie_reelle));

		(*((struct_complexe16 *) (*s_objet_resultat).objet))
				.partie_imaginaire = atan2((*((struct_complexe16 *)
				(*s_objet_argument).objet)).partie_imaginaire,
				(*((struct_complexe16 *) (*s_objet_argument).objet))
				.partie_reelle);

		if (test_cfsf(s_etat_processus, 60) == d_faux)
		{
			conversion_radians_vers_degres(&((*((struct_complexe16 *)
					(*s_objet_resultat).objet)).partie_imaginaire));
		}
	}

/*
--------------------------------------------------------------------------------
  Conversion d'un nom
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == NOM)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, ALG))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_resultat).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*s_objet_resultat).objet;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_superieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "<<");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;
		(*l_element_courant).donnee = s_objet_argument;

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 1;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_r_vers_p;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(5 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "R->P");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_inferieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, ">>");

		(*l_element_courant).suivant = NULL;
		s_objet_argument = NULL;
	}

/*
--------------------------------------------------------------------------------
  Conversion d'une expression
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument).type == ALG) ||
			((*s_objet_argument).type == RPN))
	{
		if ((s_copie_argument = copie_objet(s_etat_processus,
				s_objet_argument, 'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_copie_argument).objet;
		l_element_precedent = l_element_courant;

		while((*l_element_courant).suivant != NULL)
		{
			l_element_precedent = l_element_courant;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (((*l_element_precedent).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*(*l_element_precedent).suivant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).nombre_arguments = 1;
		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).fonction = instruction_r_vers_p;

		if (((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction =
				malloc(5 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction, "R->P");

		(*(*l_element_precedent).suivant).suivant = l_element_courant;

		s_objet_resultat = s_copie_argument;
	}

/*
--------------------------------------------------------------------------------
  Ralisation impossible de la fonction R->P
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}

// vim: ts=4
